@@ -52,6 +52,7 @@ enum {
 
 enum {
     START,
+    CONTENTS,
     PAGE,
     FINISH,
     CANCEL,
@@ -113,11 +114,10 @@ pager_class_init (YelpPagerClass *klass)
     g_object_class_install_property
 	(object_class,
 	 PROP_URI,
-	 g_param_spec_object ("uri",
-			      _("Document URI"),
-			      _("The URI of the document to be processed"),
-			      YELP_TYPE_URI,
-			      G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE));
+	 g_param_spec_pointer ("uri",
+			       _("Document URI"),
+			       _("The URI of the document to be processed"),
+			       G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE));
 
     signals[START] = g_signal_new
 	("start",
@@ -127,6 +127,14 @@ pager_class_init (YelpPagerClass *klass)
 	 yelp_marshal_VOID__VOID,
 	 G_TYPE_NONE, 0);
 
+    signals[CONTENTS] = g_signal_new
+	("contents",
+	 G_TYPE_FROM_CLASS (klass),
+	 G_SIGNAL_RUN_FIRST, 0,
+	 NULL, NULL,
+	 yelp_marshal_VOID__VOID,
+	 G_TYPE_NONE, 0);
+
     signals[PAGE] = g_signal_new
 	("page",
 	 G_TYPE_FROM_CLASS (klass),
@@ -173,7 +181,7 @@ pager_init (YelpPager *pager)
     pager->priv = priv;
 
     pager->priv->uri   = NULL;
-    pager->priv->state = YELP_PAGER_STATE_NEW;
+    pager->priv->state = 0;
 
     pager->priv->error = NULL;
     pager->priv->page_hash =
@@ -194,9 +202,13 @@ pager_set_property (GObject      *object,
     switch (prop_id) {
     case PROP_URI:
 	if (pager->priv->uri)
-	    g_object_unref (pager->priv->uri);
-	pager->priv->uri = (YelpURI *) g_value_get_object (value);
-	g_object_ref (pager->priv->uri);
+	    yelp_uri_unref (pager->priv->uri);
+
+	pager->priv->uri = (YelpURI *) g_value_get_pointer (value);
+
+	if (pager->priv->uri)
+	    yelp_uri_ref (pager->priv->uri);
+
 	break;
     default:
 	break;
@@ -213,7 +225,7 @@ pager_get_property (GObject      *object,
 
     switch (prop_id) {
     case PROP_URI:
-	g_value_set_object (value, pager->priv->uri);
+	g_value_set_pointer (value, pager->priv->uri);
 	break;
     default:
 	break;
@@ -225,7 +237,7 @@ pager_dispose (GObject *object)
 {
     YelpPager *pager = YELP_PAGER (object);
 
-    g_object_unref (pager->priv->uri);
+    yelp_uri_unref (pager->priv->uri);
 
     if (pager->priv->error)
 	g_error_free (pager->priv->error);
@@ -242,13 +254,17 @@ pager_dispose (GObject *object)
 gboolean
 yelp_pager_start (YelpPager *pager)
 {
+    YelpPagerState state;
+
     g_return_val_if_fail (pager != NULL, FALSE);
     g_return_val_if_fail (YELP_IS_PAGER (pager), FALSE);
 
-    switch (yelp_pager_get_state (pager)) {
-    case YELP_PAGER_STATE_NEW:
-    case YELP_PAGER_STATE_CANCEL:
-	yelp_pager_set_state (pager, YELP_PAGER_STATE_START);
+    state = yelp_pager_get_state (pager);
+    if (!(state & YELP_PAGER_STATE_STARTED) ||
+	 (state & YELP_PAGER_STATE_STOPPED)) {
+
+	yelp_pager_clear_state (pager);
+	yelp_pager_set_state   (pager, YELP_PAGER_STATE_STARTED);
 
 	g_object_ref (pager);
 	g_signal_emit (pager, signals[START], 0);
@@ -258,8 +274,7 @@ yelp_pager_start (YelpPager *pager)
 
 	g_object_unref (pager);
 	return TRUE;
-
-    default:
+    } else {
 	g_object_unref (pager);
 	return FALSE;
     }
@@ -271,14 +286,16 @@ yelp_pager_cancel (YelpPager *pager)
     g_return_if_fail (pager != NULL);
     g_return_if_fail (YELP_IS_PAGER (pager));
 
+    yelp_pager_set_state (pager, YELP_PAGER_STATE_STOPPED);
+
     YELP_PAGER_GET_CLASS (pager)->cancel (pager);
 }
 
 YelpURI *
 yelp_pager_get_uri (YelpPager *pager)
 {
-    g_return_val_if_fail (pager != NULL, FALSE);
-    g_return_val_if_fail (YELP_IS_PAGER (pager), FALSE);
+    g_return_val_if_fail (pager != NULL, NULL);
+    g_return_val_if_fail (YELP_IS_PAGER (pager), NULL);
 
     return pager->priv->uri;
 }
@@ -292,13 +309,22 @@ yelp_pager_get_state (YelpPager *pager)
     return pager->priv->state;
 }
 
+void
+yelp_pager_clear_state (YelpPager *pager)
+{
+    g_return_if_fail (pager != NULL);
+    g_return_if_fail (YELP_IS_PAGER (pager));
+
+    pager->priv->state = 0;
+}
+
 void
 yelp_pager_set_state (YelpPager *pager, YelpPagerState state)
 {
     g_return_if_fail (pager != NULL);
     g_return_if_fail (YELP_IS_PAGER (pager));
 
-    pager->priv->state = state;
+    pager->priv->state = pager->priv->state | state;
 }
 
 GError *
@@ -324,7 +350,7 @@ yelp_pager_error (YelpPager *pager, GError *error)
 	g_error_free (pager->priv->error);
     pager->priv->error = error;
 
-    yelp_pager_set_state (pager, YELP_PAGER_STATE_ERROR);
+    yelp_pager_set_state (pager, YELP_PAGER_STATE_STOPPED);
 
     g_signal_emit_by_name (pager, "error");
 }
@@ -338,6 +364,16 @@ yelp_pager_get_sections (YelpPager *pager)
     return YELP_PAGER_GET_CLASS (pager)->get_sections (pager);
 }
 
+gchar *
+yelp_pager_resolve_uri (YelpPager *pager, YelpURI *uri)
+{
+    gchar *frag_id = NULL;
+
+    frag_id = (gchar *) (YELP_PAGER_GET_CLASS (pager)->resolve_uri (pager, uri));
+
+    return frag_id;
+}
+
 gboolean
 yelp_pager_uri_is_page (YelpPager *pager, gchar *page_id, YelpURI *uri)
 {
@@ -375,9 +411,7 @@ yelp_pager_lookup_page (YelpPager *pager, YelpURI *uri)
     page_id = (gchar *) (YELP_PAGER_GET_CLASS (pager)->resolve_uri (pager, uri));
 
     if (!page_id)
-	page_id = yelp_uri_get_fragment (uri);
-    else
-	page_id = g_strdup (page_id);
+	page_id = g_strdup (gnome_vfs_uri_get_fragment_identifier (uri->uri));
 
     page = (YelpPage *) yelp_pager_get_page (pager, page_id);
 
@@ -401,22 +435,14 @@ yelp_pager_get_page (YelpPager *pager, gchar *id)
 
 void 
 yelp_pager_add_page (YelpPager *pager,
-		     gchar     *id,
-		     gchar     *title,
-		     gchar     *chunk)
+		     YelpPage  *page)
 {
-    YelpPage *page;
-
     g_return_if_fail (pager != NULL);
     g_return_if_fail (YELP_IS_PAGER (pager));
 
-    page = g_new0 (YelpPage, 1);
+    g_return_if_fail (page->id != NULL);
 
-    page->id    = id;
-    page->title = title;
-    page->chunk = chunk;
-
-    g_hash_table_insert (pager->priv->page_hash, id, page);
+    g_hash_table_insert (pager->priv->page_hash, page->id, page);
 }
 
 void
@@ -426,5 +452,9 @@ yelp_page_free (YelpPage *page)
     g_free (page->title);
     g_free (page->chunk);
 
+    g_free (page->prev);
+    g_free (page->next);
+    g_free (page->toc);
+
     g_free (page);
 }
